@@ -6,7 +6,7 @@ if [ "$2" = "2" ]; then
 else
     echo "run python3-nxstools"
     if [ "$1" = "debian10" ] || [ "$1" = "ubuntu22.04" ] || [ "$1" = "ubuntu20.04" ] || [ "$1" = "ubuntu20.10" ] || [ "$1" = "debian11" ] ; then
-	docker exec ndts python3 -m pytest
+	docker exec ndts python3 -m pytest --cov=nxstools --cov-report=term-missing test
     else
 	docker exec ndts python3 test
     fi
