@@ -31,12 +31,13 @@ import datetime
 import pwd
 import grp
 import fnmatch
+import yaml
 
 from .nxsparser import TableTools
 from .nxsfileparser import (NXSFileParser, numpyEncoder)
 from .nxsargparser import (Runner, NXSArgParser, ErrorException)
 from . import filewriter
-
+from .ontology import id_techniques, nexus_panet
 
 WRITERS = {}
 try:
@@ -301,12 +302,12 @@ class BeamtimeLoader(object):
     btmdmap = {
         "principalInvestigator": "pi.email",
         # "pid": "beamtimeId",   # ?? is not unique for dataset
-        "owner": "applicant.lastname",
-        "contactEmail": "contact",
+        "owner": "leader.lastname",
+        "contactEmail": "pi.email",
         "sourceFolder": "corePath",
 
         "endTime": "eventEnd",    # ?? should be endTime for dataset
-        "ownerEmail": "applicant.email",
+        "ownerEmail": "leader.email",
         "description": "title",   # ?? should be from dataset
         "createdAt": "generated",  # ?? should be automatic
         "updatedAt": "generated",  # ?? should be automatic
@@ -315,13 +316,15 @@ class BeamtimeLoader(object):
     }
 
     strcre = {
-        "creationLocation": "/DESY/{facility}/{beamline}",
+        "creationLocation": "/DESY/{facility}/{beamlineAlias}",
         "type": "raw",
         "isPublished": False,
-        "ownerGroup": "{beamtimeId}-part",
-        "accessGroups": ["{beamtimeId}-clbt",
-                         "{beamtimeId}-dmgt",
-                         "{beamline}dmgt"]
+        "ownerGroup": "{beamtimeId}-dmgt",
+        "accessGroups": ["{beamtimeId}-dmgt",
+                         "{beamtimeId}-clbt",
+                         "{beamtimeId}-part",
+                         "{beamline}dmgt",
+                         "{beamline}staff"]
     }
 
     cre = {
@@ -533,8 +536,8 @@ class BeamtimeLoader(object):
             else:
                 yield (key, dct2[key])
 
-    def updatepid(self, metadata, filename=None, puuid=False, pfname=False,
-                  beamtimeid=None):
+    def update_pid(self, metadata, filename=None, puuid=False, pfname=False,
+                   beamtimeid=None):
         """ update pid metadata with dictionary
 
         :param metadata: metadata dictionary to merge in
@@ -611,6 +614,119 @@ class BeamtimeLoader(object):
                 metadata["datasetName"] = metadata["pid"]
         return metadata
 
+    def update_techniques(self, metadata, techniques=None):
+        """ update techniques
+
+        :param metadata: metadata dictionary to merge in
+        :type metadata: :obj:`dict` <:obj:`str`, `any`>
+        :param techniques: a list of techniques splitted by comma
+        :type techniques: :obj:`str`
+        :returns: metadata dictionary
+        :rtype: :obj:`dict` <:obj:`str`, `any`>
+        """
+        if techniques:
+            metadata["techniques"] = \
+                self.generate_techniques(techniques.split(","))
+        if metadata and "techniques" not in metadata:
+            try:
+                if "scientificMetadata" in metadata and \
+                       "definition" in metadata["scientificMetadata"]:
+                    gdefin = metadata["scientificMetadata"]["definition"]
+                    if "value" in gdefin:
+                        defin = gdefin["value"].split(",")
+                        defin = [
+                            (df[2:]
+                             if (df and str(df).startswith("NX")) else df)
+                            for df in defin]
+                        if defin:
+                            metadata["techniques"] = \
+                                self.generate_techniques(defin)
+            except Exception as e:
+                sys.stderr.write("nxsfileinfo: '%s'\n"
+                                 % str(e))
+
+            try:
+                if "scientificMetadata" in metadata and \
+                       "experiment_description" in \
+                       metadata["scientificMetadata"]:
+                    gexpdes = \
+                        metadata["scientificMetadata"][
+                            "experiment_description"]
+                    if "value" in gexpdes:
+                        try:
+                            pids = None
+                            tes = None
+                            expdes = yaml.safe_load(gexpdes["value"])
+                            if "techniques" in expdes:
+                                tes = expdes["techniques"]
+                            elif "technique" in expdes:
+                                tes = [expdes["technique"]]
+                            if "techniques_pids" in expdes:
+                                pids = expdes["techniques_pids"]
+                            elif "technique_pid" in expdes:
+                                pids = [expdes["technique_pid"]]
+                            if tes is not None and pids is not None:
+                                metadata["techniques"] = \
+                                    self.generate_techniques(tes, pids)
+                            elif tes is not None:
+                                metadata["techniques"] = \
+                                    self.generate_techniques(tes)
+                            elif pids is not None:
+                                metadata["techniques"] = \
+                                    self.generate_techniques(pids)
+                            else:
+                                metadata["techniques"] = \
+                                    self.generate_techniques(
+                                        [gexpdes["value"]])
+                        except Exception:
+                            metadata["techniques"] = \
+                                self.generate_techniques(
+                                    [gexpdes["value"]])
+            except Exception as e:
+                sys.stderr.write("nxsfileinfo: '%s'\n"
+                                 % str(e))
+
+        if metadata and "techniques" not in metadata:
+            metadata["techniques"] = []
+        return metadata
+
+    def generate_techniques(self, techniques, techniques_pids=None):
+        """ generate technique dictionary
+
+        :param techniques: a list of techniques splitted by comma
+        :type techniques: :obj:`list` <:obj:`str`>
+        :param techniques_pids: a list of technique pids splitted by comma
+        :type techniques_pids: :obj:`list` <:obj:`str`>
+        :returns: technique dictionary
+        :rtype: :obj:`dict` <:obj:`str`, `objstr`>
+        """
+        result = []
+        # print(techniques)
+        # print(techniques_pids)
+        for it, te in enumerate(techniques):
+            pid = None
+            name = te
+            if techniques_pids and len(techniques_pids) > it and \
+               techniques_pids[it] is not None:
+                pid = techniques_pids[it]
+            elif te.startswith("http:/") and te in id_techniques.keys():
+                pid = te
+                name = id_techniques[pid]
+            elif te in nexus_panet.keys():
+                pid = nexus_panet[te]
+                name = id_techniques[pid]
+            elif te.startswith("PaNET"):
+                nm = "http://purl.org/pan-science/PaNET/%s" % te
+                if nm in id_techniques.keys():
+                    pid = nm
+                    name = id_techniques[pid]
+            if pid:
+                result.append({"pid": pid, "name": name})
+            else:
+                result.append({"name": name})
+        # print(result)
+        return result
+
 
 class Metadata(Runner):
 
@@ -636,14 +752,14 @@ class Metadata(Runner):
             "-a", "--attributes",
             help="names of field or group attributes to be show "
             " (separated by commas without spaces). "
-            "The  default takes all attributes",
+            "The default takes all attributes",
             dest="attrs", default=None)
         self._parser.add_argument(
             "-n", "--hidden-attributes",
             help="names of field or group attributes to be hidden "
             " (separated by commas without spaces). "
-            "The  default: 'nexdatas_source,nexdatas_strategy'",
-            dest="nattrs", default="nexdatas_source,nexdatas_strategy")
+            "The default: 'nexdatas_source,nexdatas_strategy,units'",
+            dest="nattrs", default="nexdatas_source,nexdatas_strategy,units")
         self._parser.add_argument(
             "-v", "--values",
             help="field names of more dimensional datasets"
@@ -653,35 +769,46 @@ class Metadata(Runner):
         self._parser.add_argument(
             "-w", "--owner-group",
             default="", dest="ownergroup",
-            help="owner group name. Default is {beamtimeid}-part")
+            help="owner group name. Default is {beamtimeid}-dmgt")
         self._parser.add_argument(
             "-c", "--access-groups",
             default=None, dest="accessgroups",
             help="access group names separated by commas. "
-            "Default is {beamtimeid}-clbt,{beamtimeId}-dmgt,{beamline}dmgt")
+            "Default is {beamtimeId}-dmgt,{beamtimeid}-clbt,{beamtimeId}-part,"
+            "{beamline}dmgt,{beamline}staff")
         self._parser.add_argument(
             "-g", "--group-postfix",
             help="postfix to be added to NeXus group name. "
-            "The  default: ''",
+            "The default: ''",
             dest="group_postfix", default="")
         self._parser.add_argument(
             "-t", "--entry-classes",
             help="names of entry NX_class to be shown"
             " (separated by commas without spaces)."
             " If name is '' all groups are shown. "
-            "The  default: 'NXentry'",
+            "The default: 'NXentry'",
             dest="entryclasses", default="NXentry")
         self._parser.add_argument(
             "-e", "--entry-names",
             help="names of entry groups to be shown"
             " (separated by commas without spaces)."
             " If name is '' all groups are shown. "
-            "The  default: ''",
+            "The default: ''",
             dest="entrynames", default="")
+        self._parser.add_argument(
+            "-q", "--techniques",
+            help="names of techniques"
+            " (separated by commas without spaces)."
+            "The default: ''",
+            dest="techniques", default="")
         self._parser.add_argument(
             "-m", "--raw-metadata", action="store_true",
             default=False, dest="rawscientific",
             help="do not store NXentry as scientificMetadata")
+        self._parser.add_argument(
+            "--oned", action="store_true",
+            default=False, dest="oned",
+            help="add 1d values to scientificMetadata")
         self._parser.add_argument(
             "-p", "--pid", dest="pid",
             help=("dataset pid"))
@@ -708,6 +835,9 @@ class Metadata(Runner):
             "--h5cpp", action="store_true",
             default=False, dest="h5cpp",
             help="use h5cpp module as a nexus reader")
+        self._parser.add_argument(
+            "-x", "--chmod", dest="chmod",
+            help=("json metadata file mod bits, e.g. 0o662"))
 
     def postauto(self):
         """ parser creator after autocomplete run """
@@ -815,15 +945,22 @@ class Metadata(Runner):
             nxsparser.scientific = not options.rawscientific
             nxsparser.attrs = attrs
             nxsparser.hiddenattrs = nattrs
+            if hasattr(options, "oned"):
+                nxsparser.oned = options.oned
             nxsparser.parseMeta()
 
         if nxsparser is None:
             bl = BeamtimeLoader(options)
             result = bl.run()
             result = bl.overwrite(result)
-            result = bl.updatepid(
+            result = bl.update_pid(
                 result, None, options.puuid,
                 options.pfname, options.beamtimeid)
+            if not options.rawscientific:
+                techniques = None
+                if hasattr(options, "techniques"):
+                    techniques = options.techniques
+                result = bl.update_techniques(result, techniques)
         elif nxsparser and nxsparser.description:
             if len(nxsparser.description) == 1:
                 desc = nxsparser.description[0]
@@ -845,9 +982,14 @@ class Metadata(Runner):
                 bl.run()
                 result = bl.merge(desc)
                 result = bl.overwrite(result)
-                result = bl.updatepid(
+                result = bl.update_pid(
                     result, options.args[0], options.puuid,
                     options.pfname, options.beamtimeid)
+                if not options.rawscientific:
+                    techniques = None
+                    if hasattr(options, "techniques"):
+                        techniques = options.techniques
+                    result = bl.update_techniques(result, techniques)
 
             else:
                 result = []
@@ -870,9 +1012,14 @@ class Metadata(Runner):
                     bl.run()
                     rst = bl.merge(desc)
                     rst = bl.overwrite(rst)
-                    rst = bl.updatepid(
+                    rst = bl.update_pid(
                         rst, options.args[0], options.puuid,
                         options.pfname, options.beamtimeid)
+                    if not options.rawscientific:
+                        techniques = None
+                        if hasattr(options, "techniques"):
+                            techniques = options.techniques
+                        rst = bl.update_techniques(rst, techniques)
                     result.append(rst)
         if result is not None:
             return json.dumps(
@@ -891,8 +1038,30 @@ class Metadata(Runner):
             metadata = self.metadata(root, options)
             if metadata:
                 if options.output:
-                    with open(options.output, "w") as fl:
-                        fl.write(metadata)
+                    chmod = None
+                    try:
+                        chmod = int(options.chmod, 8)
+                    except Exception:
+                        options.chmod = None
+
+                    if options.chmod:
+                        oldmask = os.umask(0)
+
+                        def opener(path, flags):
+                            return os.open(path, flags, chmod)
+
+                        try:
+                            with open(options.output,
+                                      "w", opener=opener) as fl:
+                                fl.write(metadata)
+                        except Exception:
+                            with open(options.output, "w") as fl:
+                                fl.write(metadata)
+                            os.chmod(options.output, chmod)
+                        os.umask(oldmask)
+                    else:
+                        with open(options.output, "w") as fl:
+                            fl.write(metadata)
                 else:
                     print(metadata)
         except Exception as e:
@@ -925,12 +1094,13 @@ class OrigDatablock(Runner):
         self._parser.add_argument(
             "-w", "--owner-group",
             default="", dest="ownergroup",
-            help="owner group name. Default is {beamtimeid}-part")
+            help="owner group name. Default is {beamtimeid}-dmgt")
         self._parser.add_argument(
             "-c", "--access-groups",
             default=None, dest="accessgroups",
             help="access group names separated by commas. "
-            "Default is {beamtimeid}-clbt,{beamtimeId}-dmgt")
+            "Default is {beamtimeId}-dmgt,{beamtimeid}-clbt,{beamtimeId}-part,"
+            "{beamline}dmgt,{beamline}staff")
         self._parser.add_argument(
             "-s", "--skip",
             help="filters for files to be skipped (separated by commas "
@@ -941,11 +1111,14 @@ class OrigDatablock(Runner):
             help="list of filtes to be added (separated by commas "
             "without spaces). Default: ''. E.g. 'scan1.nxs,scan2.nxs'",
             dest="add", default="")
+        self._parser.add_argument(
+            "-x", "--chmod", dest="chmod",
+            help=("json metadata file mod bits, e.g. 0o662"))
 
     def postauto(self):
         """ parser creator after autocomplete run """
         self._parser.add_argument(
-            'args', metavar='scan_name', type=str, nargs=1,
+            'args', metavar='scan_name', type=str, nargs='+',
             help='scan name')
         self._parser.add_argument(
             "-o", "--output", dest="output",
@@ -1013,7 +1186,7 @@ class OrigDatablock(Runner):
                 path = path[2:]
             if relpath:
                 path = os.path.join(relpath, path)
-            rec["path"] = path
+            rec["path"] = os.path.normpath(path)
             rec["size"] = status.st_size
             rec["time"] = self.isotime(status.st_ctime)
             try:
@@ -1050,32 +1223,41 @@ class OrigDatablock(Runner):
         scanfiles = []
         scandirs = []
         totsize = 0
+        fscandir = None
 
-        scandir, scanname = os.path.split(os.path.abspath(options.args[0]))
-        for (dirpath, dirnames, filenames) in os.walk(scandir):
-            scanfiles = [f for f in filenames if f.startswith(scanname)]
-            scandirs = [f for f in dirnames if f.startswith(scanname)]
-            break
-        flist, tsize = self.datafiles(scandir, "", scanfiles,
-                                      options.relpath, skip)
-        dtfiles.extend(flist)
-        totsize += tsize
+        for arg in options.args:
+            scandir, scanname = os.path.split(os.path.abspath(arg))
+            if not fscandir:
+                fscandir = fscandir or scandir
+                relpath = options.relpath
+            else:
+                relpath = os.path.relpath(scandir, fscandir)
+                if options.relpath:
+                    relpath = os.path.join(options.relpath, relpath)
+            for (dirpath, dirnames, filenames) in os.walk(scandir):
+                scanfiles = [f for f in filenames if f.startswith(scanname)]
+                scandirs = [f for f in dirnames if f.startswith(scanname)]
+                break
+            flist, tsize = self.datafiles(scandir, "", scanfiles,
+                                          relpath, skip)
+            dtfiles.extend(flist)
+            totsize += tsize
 
-        for fl in add:
-            if os.path.isfile(fl):
-                ascandir, ascanname = os.path.split(os.path.abspath(fl))
-                flist, tsize = self.datafiles(
-                    scandir, ascandir, [ascanname], options.relpath)
-                dtfiles.extend(flist)
-                totsize += tsize
+            for fl in add:
+                if os.path.isfile(fl):
+                    ascandir, ascanname = os.path.split(os.path.abspath(fl))
+                    flist, tsize = self.datafiles(
+                        scandir, ascandir, [ascanname], relpath)
+                    dtfiles.extend(flist)
+                    totsize += tsize
 
-        for scdir in scandirs:
-            for (dirpath, dirnames, filenames) in os.walk(
-                    os.path.join(scandir, scdir)):
-                flist, tsize = self.datafiles(
-                    scandir, dirpath, filenames, options.relpath, skip)
-                dtfiles.extend(flist)
-                totsize += tsize
+            for scdir in scandirs:
+                for (dirpath, dirnames, filenames) in os.walk(
+                        os.path.join(scandir, scdir)):
+                    flist, tsize = self.datafiles(
+                        scandir, dirpath, filenames, relpath, skip)
+                    dtfiles.extend(flist)
+                    totsize += tsize
         result["dataFileList"] = dtfiles
         result["size"] = totsize
         if options.ownergroup:
@@ -1089,7 +1271,7 @@ class OrigDatablock(Runner):
            and result["datasetId"] and \
            len(result["datasetId"].split("/")) > 1:
             if "ownerGroup" not in result:
-                result["ownerGroup"] = "%s-part" % (
+                result["ownerGroup"] = "%s-dmgt" % (
                     result["datasetId"].split("/")[1])
             if "accessGroups" not in result:
                 result["accessGroups"] = [
@@ -1111,8 +1293,29 @@ class OrigDatablock(Runner):
             metadata = self.datablock(options)
             if metadata:
                 if options.output:
-                    with open(options.output, "w") as fl:
-                        fl.write(metadata)
+                    chmod = None
+                    try:
+                        chmod = int(options.chmod, 8)
+                    except Exception:
+                        options.chmod = None
+
+                    if options.chmod:
+                        oldmask = os.umask(0)
+
+                        def opener(path, flags):
+                            return os.open(path, flags, chmod)
+                        try:
+                            with open(options.output,
+                                      "w", opener=opener) as fl:
+                                fl.write(metadata)
+                        except Exception:
+                            with open(options.output, "w") as fl:
+                                fl.write(metadata)
+                            os.chmod(options.output, chmod)
+                        os.umask(oldmask)
+                    else:
+                        with open(options.output, "w") as fl:
+                            fl.write(metadata)
                 else:
                     print(metadata)
         except Exception as e:
