@@ -193,12 +193,15 @@ class NXSFileParser(object):
             "source_type": ["nexdatas_source", getdstype],
             "source": ["nexdatas_source", getdssource],
             "strategy": ["nexdatas_strategy", str],
+            "unit": ["units", str],
         }
         #: (:obj:`list`< :obj:`str`>)  field names which value should be stored
         self.valuestostore = ["depends_on"]
         self.__root = root
         #: (:obj:`list`< :obj:`str`>)  filters for `full_path` names
         self.filters = []
+        # (:obj:`bool`) oned value flag
+        self.oned = False
 
     @classmethod
     def getpath(cls, path):
@@ -243,8 +246,12 @@ class NXSFileParser(object):
                 cont = True
                 while cont:
                     try:
-                        if not isinstance(vl, str) and \
-                           (hasattr(vl, "__len__") and len(vl) == 1):
+                        if isinstance(vl, np.ndarray) and \
+                           vl.shape == ():
+                            vl = vl.item()
+                            cont = False
+                        elif not isinstance(vl, str) and \
+                                (hasattr(vl, "__len__") and len(vl) == 1):
                             vl = vl[0]
                         else:
                             cont = False
@@ -449,15 +456,21 @@ class NXSFileParser(object):
         if not isinstance(node, filewriter.FTGroup):
             if (node.name in self.valuestostore and node.is_valid) \
                or "shape" not in desc \
-               or desc["shape"] in [None, [1], []]:
+               or desc["shape"] in [None, [1], []] \
+               or (self.oned and len(desc["shape"]) == 1):
                 if hasattr(node, "read"):
                     try:
                         vl = node.read()
                         cont = True
                         while cont:
                             try:
-                                if not isinstance(vl, str) and \
-                                   (hasattr(vl, "__len__") and len(vl) == 1):
+                                if isinstance(vl, np.ndarray) and \
+                                   vl.shape == ():
+                                    vl = vl.item()
+                                    cont = False
+                                elif not isinstance(vl, str) and \
+                                        (hasattr(vl, "__len__") and
+                                         len(vl) == 1):
                                     vl = vl[0]
                                 else:
                                     cont = False
