@@ -933,6 +933,14 @@ standardComponentVariables = {
             'doc': "data axes field name(s) (string)"
         },
     },
+    'descriptiontext': {
+        'description': {
+            'default': "scan",
+            'doc': "scan technique (string)"
+        },
+    },
+    'description': {
+    },
     'defaultinstrument': {
         'control': {
             'default': None,
@@ -1429,6 +1437,13 @@ standardComponentTemplateFiles = {
         'beamtimefname.ds.xml',
         'start_time.ds.xml',
     ],
+    'description': [
+        'description.xml',
+        'experiment_description.ds.xml',
+    ],
+    'descriptiontext': [
+        'descriptiontext.xml',
+    ],
     'default': [
         'default.xml',
         'defaultsample.xml',
