@@ -302,7 +302,8 @@ class NXSCreateStdCompFSTest(unittest.TestCase):
                      "collect4", "collect5", "collect6",
                      "common2", "common3",
                      "datasignal", "dcm", "default", "defaultinstrument",
-                     "defaultsample", "detectorlive",
+                     "defaultsample", "description",
+                     "descriptiontext", "detectorlive",
                      "empty", "groupsecop", "keithley", "maia",
                      "maiadimension", "maiaflux", "msnsar", "mssar",
                      "pinhole",
@@ -324,7 +325,8 @@ class NXSCreateStdCompFSTest(unittest.TestCase):
                  "collect4", "collect5", "collect6",
                  "common2", "common3",
                  "datasignal", "dcm", "default", "defaultinstrument",
-                 "defaultsample", "detectorlive",
+                 "defaultsample", "description",
+                 "descriptiontext", "detectorlive",
                  "empty", "groupsecop", "keithley", "maia",
                  "maiadimension", "maiaflux", "pinhole",
                  "pointdet", "qbpm", "samplehkl",
